//  cssutil.c - utility for munging css files, version X0.1
//  Copyright 2001  William S. Yerazunis, all rights reserved.
//  
//  This software is licensed to the public under the Free Software
//  Foundation's GNU GPL, version 1.0.  You may obtain a copy of the
//  GPL by visiting the Free Software Foundations web site at
//  www.fsf.org .  Other licenses may be negotiated; contact the 
//  author for details.  
//

//  include some standard files

#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <dirent.h>
#include <regex.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

//  include any local crm114 configuration file
#include "crm114_config.h"

//  include the crm114 data structures file
#include "crm114_structs.h"

//  and include the routine declarations file
#include "crm114.h"

int main (int argc, char **argv)
{

  long i,j,k;    //  some random counters, when we need a loop
  long hfsize;
  long sum; // sum of the hits... can be _big_.  
  FILE *f;
  int hfd1, hfd2;

  {
	//     learn the sparse spectrum of this input window as 
	//     belonging to a particular type.
	//     learn <flags> (classname) /word/
	//
    struct stat statbuf;    //  filestat buffer
    unsigned char *h1, *h2;              //  the text of the hash file
    //   filename is argv [1]
    //             and stat it to get it's length
    k = stat (argv[1], &statbuf);
    //             quick check- does the file even exist?
    if (k != 0)
      {
	//      file didn't exist... create it
	fprintf (stderr, "\nHad to create .CSS file %s", argv[1]);
	f = fopen (argv[1], "w");
	//       put in  bytes of NULL
	for (j = 0; j < DEFAULT_SPARSE_SPECTRUM_FILE_LENGTH; j++) 
	  fprintf (f,"%c", '\000');
	fclose (f);
	//    and reset the statbuf to be correct
	    k = stat (argv[1], &statbuf);
      };
    //    
    hfsize = statbuf.st_size;
    fprintf (stderr, "Sparse spectra file %s has %ld bins total\n",
	     argv[1], hfsize);
    //         mmap the hash file into memory so we can bitwhack it
    hfd1 = open (argv[1], O_RDWR);
    h1 = (unsigned char *) mmap (NULL, hfsize, 
				     PROT_READ + PROT_WRITE,
				     MAP_SHARED, hfd1, 0);
    //
    
    //  and repeat the process for the second file:
    k = stat (argv[2], &statbuf);
    //             quick check- does the file even exist?
    if (k != 0)
      {
	//      file didn't exist... create it
	fprintf (stderr, "\nHad to create .CSS file %s", argv[2]);
	f = fopen (argv[2], "w");
	//       put in some  bytes of NULL
	for (j = 0; j < DEFAULT_SPARSE_SPECTRUM_FILE_LENGTH; j++) 
	  fprintf (f,"%c", '\000');
	fclose (f);
	//    and reset the statbuf to be correct
	    k = stat (argv[2], &statbuf);
      };
    //    
    hfsize = statbuf.st_size;
    fprintf (stderr, "Sparse spectra file %s has %ld bins total\n",
	     argv[2], hfsize);
    //         mmap the hash file into memory so we can bitwhack it
    hfd2 = open (argv[2], O_RDWR);
    h2 = (unsigned char *) mmap (NULL, hfsize, 
				     PROT_READ + PROT_WRITE,
				     MAP_SHARED, hfd2, 0);



    sum = 0;
    for ( i = 0; i < hfsize; i++) 
      {
	j = h1[i] ;
	if (j > h2[i] ) j = h2[i];
	sum = sum + j;
      };
    fprintf (stderr, "\n Similarities between files: %ld\n", sum);
    
    sum = 0;
    for ( i = 0; i < hfsize; i++) 
      {
	j = h1[i] - h2[i];
	if (j < 0) j = -j;
	sum = sum + j;
      };
    fprintf (stderr, "\n Difference between files: %ld\n", sum);

    sum = 0;
    for ( i = 0; i < hfsize; i++) 
      {
	j = h1[i] - h2[i];
	if (j < 0) j = 0;
	sum = sum + j;
      };
    fprintf (stderr, "\n File 1 dominates: %ld\n", sum);
    
    sum = 0;
    for ( i = 0; i < hfsize; i++) 
      {
	j = h2[i] - h1[i];
	if (j < 0) j = 0;
	sum = sum + j;
      };
    fprintf (stderr, "\n File 2 dominates: %ld\n", sum);
    
  }
  return 0;
}
