//  cssutil.c - utility for munging css files, version X0.1
//  Copyright 2001  William S. Yerazunis, all rights reserved.
//  
//  This software is licensed to the public under the Free Software
//  Foundation's GNU GPL, version 1.0.  You may obtain a copy of the
//  GPL by visiting the Free Software Foundations web site at
//  www.fsf.org .  Other licenses may be negotiated; contact the 
//  author for details.  
//

//  include some standard files

#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <dirent.h>
#include <regex.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>

//  include any local crm114 configuration file
#include "crm114_config.h"

//  include the crm114 data structures file
#include "crm114_structs.h"

//  and include the routine declarations file
#include "crm114.h"

int main (int argc, char **argv)
{

  long i,j,k;    //  some random counters, when we need a loop
  long v;
  long hfsize;
  long long sum; // sum of the hits... can be _big_.  
  FILE *f;
  int hfd;

  char cmdstr[255];
  char cmdchr [2];
  char crapchr [2];
  float cmdval;

  //    the following for crm114.h's happiness

  char * newinputbuf;
  newinputbuf = (char *) &hfd;
  
  {
	//     learn the sparse spectrum of this input window as 
	//     belonging to a particular type.
	//     learn <flags> (classname) /word/
	//
    struct stat statbuf;    //  filestat buffer
    unsigned char *hashes;              //  the text of the hash file
    //   filename is argv [1]
    //             and stat it to get it's length
    k = stat (argv[1], &statbuf);
    //             quick check- does the file even exist?
    if (k != 0)
      {
	//      file didn't exist... create it
	fprintf (stdout, "\nHad to create .CSS file %s", argv[1]);
	f = fopen (argv[1], "w");
	//       put in some bytes of NULL
	for (j = 0; j < DEFAULT_SPARSE_SPECTRUM_FILE_LENGTH; j++) 
	  fprintf (f,"%c", '\000');
	fclose (f);
	//    and reset the statbuf to be correct
	    k = stat (argv[1], &statbuf);
      };
    //    
    hfsize = statbuf.st_size;
    fprintf (stdout, "Sparse spectra file %s has %ld bins total\n",
	     argv[1], hfsize);
    //         mmap the hash file into memory so we can bitwhack it
    hfd = open (argv[1], O_RDWR);
    hashes = (unsigned char *) mmap (NULL, hfsize, 
				     PROT_READ + PROT_WRITE,
				     MAP_SHARED, hfd, 0);
    //
    
  zloop:
    sum = 0;
    for ( i = 0; i < hfsize; i++) 
      sum = sum + hashes [i];
    fprintf (stdout, "\n total number of hash datums in this file is %lld",
	     sum);

    fprintf (stdout, "\n now scanning bins- please be patient...");

    for (v = 0; v <= 255; v++)
      {
	sum = 0;
	for (i = 0; i < hfsize; i++)
	  {
	    if (hashes[i] == v) 
	      {
		sum++;
		//  if (v > 0) fprintf (stdout, " %ld", i);
	      }
	  }
	if (v == 0 || sum != 0) 
	  fprintf (stdout, "\n bin value %ld found %lld times", v, sum);
      }
    
  cmdloop:
    fprintf (stdout, "\nOptions:\n");
    fprintf (stdout, "   Z n - zero bins at or below a value\n");
    fprintf (stdout, "   S n - subtract a constant from all bins\n");
    fprintf (stdout, "   D n - divide all bins by a constant\n");
    fprintf (stdout, "   R - rescan\n");
    fprintf (stdout, "   Q - quit\n");
    fprintf (stdout, ">>> ");

    clearerr (stdin);
    fscanf (stdin, "%[^\n]", cmdstr);
    fscanf (stdin, "%c", crapchr);
    sscanf (cmdstr, "%s %f", cmdchr, &cmdval);
    if (strcasecmp (cmdchr, "z") == 0)
      {
	fprintf (stdout, "Working...");
	for (i = 0; i < hfsize; i++)
	  if (hashes[i] <= cmdval) hashes[i] = 0;
	fprintf (stdout, "done.\n");
	goto cmdloop;
      };
    if (strcasecmp (cmdchr, "s") == 0)
      {
	fprintf (stdout, "Working...");
	for (i = 0; i < hfsize; i++)
	  {
	    if (hashes[i] > (int) cmdval) 
	      {
		hashes[i] = hashes[i] - cmdval;
	      }
	    else
	      {
		hashes[i] = 0;
	      };
	  };
	fprintf (stdout, "done.\n");
	goto cmdloop;
      };
    if (strcasecmp (cmdchr, "d") == 0)
      {
	fprintf (stdout, "Working...");
	for (i = 0; i < hfsize; i++)
	  hashes[i] = hashes[i] / cmdval;
	fprintf (stdout, "done.\n");
	goto cmdloop;
      };
    if (strcasecmp (cmdchr, "r") == 0)
      {
	goto zloop;
      };
    if (strcasecmp (cmdchr, "q") == 0)
      {
	fprintf (stdout, "Bye! \n");
	goto done;
      };
    
  done: ;
  }
  return 0;
}
